@@ -263,13 +263,13 @@ write.cmtk.list<-function(x,con,tablevel=0){
 #'   on the command line of CMTK tools.
 #' @param x 5x3 matrix of CMTK registration parameters OR 4x4 homogeneous affine
 #'   matrix
-#' @param centre Optional centre of rotation passed to \code{affmat2cmtkparams}
+#' @param centre Optional centre of rotation passed to \code{affmat2cmtkparams} 
 #'   when decomposing 4x4 affine matrix
 #' @param reference,floating Path to refererence and floating images.
-#' @return list of registration parameters suitable for 
-#'   \code{\link{write.cmtkreg}}
+#' @return \code{list} of class \code{cmtkreg} containing registration
+#'   parameters suitable for \code{\link{write.cmtkreg}}
 #' @export
-#' @seealso \code{\link{write.cmtkreg}, \link{affmat2cmtkparams}}
+#' @seealso \code{\link{write.cmtkreg}, \link{affmat2cmtkparams}, \link{cmtkreg}}
 cmtkreglist<-function(x,centre=c(0,0,0),reference="dummy",floating="dummy"){
   
   mat.ok=FALSE
@@ -292,18 +292,50 @@ cmtkreglist<-function(x,centre=c(0,0,0),reference="dummy",floating="dummy"){
   version=attr(x,'version')
   if(is.null(version)) version=numeric_version('2.4')
   attr(l,'version')=version
-  l
+  as.cmtkreg(l)
 }
 
-#' Read and Write CMTK landmarks
+#' Extract affine registration from CMTK registration file or in-memory list
 #' 
-#' @details CMTK landmarks are always unpaired i.e. only contain information for
-#'   one brain.
-#' @param con Character vector specifying path or a connection (passed straight
-#'   to \code{read.cmtk})
+#' @param r A registration list or path to file on disk
+#' @param outdir Optional path to output file
+#' @return When \code{outdir} is missing a list containing the registration
+#'   paramers. Otherwise \code{NULL} invisibly.
+#' @family cmtk-io
+#' @seealso \code{\link{cmtkreglist}}
 #' @export
-#' @rdname cmtklandmarks
-read.cmtklandmarks<-function(con){
+cmtk.extract_affine<-function(r, outdir) {
+  f=NULL
+  if(is.character(r)) {
+    if(!file.exists(r)) stop("Can't find registration:", r)
+    f=r
+    r=read.cmtkreg(r)
+  }
+  
+  required_fields=c("reference_study", "floating_study", "affine_xform")
+  missing_fields=setdiff(required_fields, names(r$registration))
+  if(length(missing_fields))
+    stop("The registration is missing fields: ", cat(missing_fields, collapse=", "))
+  
+  r2=r
+  # set other fields to NULL
+  r2$registration[setdiff(names(r$registration), required_fields)]=NULL
+
+  version=as.character(attr(r,'version'))
+  if(!length(version)) version='2.4'
+  if(!missing(outdir))
+    write.cmtkreg(r2, foldername = outdir, version = version)
+  else r2
+}
+
+# Read and Write CMTK landmarks
+# 
+# @details CMTK landmarks are always unpaired i.e. only contain information for
+#   one brain.
+# @param con Character vector specifying path or a connection (passed straight
+#   to \code{read.cmtk})
+# @rdname cmtklandmarks
+read.landmarks.cmtk<-function(con){
   l=read.cmtk(con,CheckLabel=FALSE)
   x=t(sapply(l,function(x) x[["location"]]))
   rn=sapply(l,function(x) x[["name"]])
@@ -314,12 +346,23 @@ read.cmtklandmarks<-function(con){
   x
 }
 
-#' @description \code{cmtklandmarks} generates in memory list representation of
-#'   cmtk landmarks
-#' @param xyzs Nx3 matrix of landmarks
-#' @rdname cmtklandmarks
-#' @export
-#' @family cmtk-io
+is.cmtklandmarks<-function(f, bytes=NULL){
+  if(!is.null(bytes) && length(f)>1)
+    stop("can only supply raw bytes to check for single file")
+  if(length(f)>1) return(sapply(f, is.cmtklandmarks))
+  
+  tocheck=if(is.null(bytes)) f else bytes
+  if(!generic_magic_check(tocheck, "! TYPEDSTREAM")) return(FALSE)
+  # still not sure? Now we need to start reading in some lines
+  h=readLines(f, n = 3)
+  isTRUE(any(grepl("landmark",h, useBytes=T, fixed = T)))
+}
+
+# @description \code{cmtklandmarks} generates in memory list representation of
+#   cmtk landmarks
+# @param xyzs Nx3 matrix of landmarks
+# @rdname cmtklandmarks
+# @family cmtk-io
 cmtklandmarks<-function(xyzs){
   # IGS Landmark lists are unpaired ie contain information for only 1 brain
   xyzs=data.matrix(xyzs)
@@ -332,16 +375,10 @@ cmtklandmarks<-function(xyzs){
   ll
 }
 
-#' @param filename Path to write out cmtklandmarks
-#' @param Force Whether to overwrite an existing landmarks file (default FALSE)
-#' @rdname cmtklandmarks
-#' @export
-write.cmtklandmarks<-function(xyzs,filename,Force=FALSE){
+# @param filename Path to write out cmtklandmarks
+# @rdname cmtklandmarks
+write.landmarks.cmtk<-function(xyzs,filename){
   ll=cmtklandmarks(xyzs)
   if(file.exists(filename) && file.info(filename)$isdir) filename=file.path(filename,"landmarks")
-  if(file.exists(filename) && !Force) {
-    stop(paste(filename,"already exists, use Force=TRUE to replace"))
-  }
   write.cmtk(ll,filename)
 }
-
