@@ -3,13 +3,41 @@
 #' @details This function will handle \code{neuron} and \code{dotprops} objects 
 #'   saved in R .rds or .rda format by default. Additional file formats can be 
 #'   registered using \code{fileformats}.
+#'   
+#'   At the moment the following formats are supported using file readers 
+#'   already included with the nat package: \itemize{
+#'   
+#'   \item \bold{swc} See \code{\link{read.neuron.swc}}
+#'   
+#'   \item \bold{neuroml} See \code{\link{read.neuron.neuroml}}
+#'   
+#'   \item \bold{fijitraces} See \code{\link{read.neuron.fiji}}. The file format
+#'   used by the \href{http://fiji.sc/Simple_Neurite_Tracer}{Simple Neurite 
+#'   Tracer} plugin of Fiji/ImageJ.
+#'   
+#'   \item \bold{hxlineset,hxskel} Two distinct fileformats used by Amira. 
+#'   \code{hxlineset} is the generic one, \code{hxskel} is used by the 
+#'   hxskeletonize extension of Schmitt and Evers (see refs).
+#'   
+#'   \item \bold{rda,rds} Native R cross-platform binary formats (see 
+#'   \code{\link{load}, \link{readRDS}}). Note that RDS only contains a single 
+#'   unnamed neuron, whereas rda contains one or more named neurons.
+#'   
+#'   }
 #' @export
-#' @param f Path to file
+#' @param f Path to file. This can be a URL, in which case the file is
+#'   downloaded to a temporary location before reading.
 #' @param format The file format of the neuron. When \code{format=NULL}, the 
-#'   default, \code{read.neuron} will infer the file format from the extension
+#'   default, \code{read.neuron} will infer the file format from the extension 
 #'   or file header (aka magic) using the \code{fileformats} registry.
 #' @param ... additional arguments passed to format-specific readers
 #' @seealso \code{\link{read.neurons}, \link{fileformats}}
+#' @references Schmitt, S. and Evers, J. F. and Duch, C. and Scholz, M. and 
+#'   Obermayer, K. (2004). New methods for the computer-assisted 3-D 
+#'   reconstruction of neurons from confocal image stacks. Neuroimage 4, 
+#'   1283--98. 
+#'   \href{http://dx.doi.org/10.1016/j.neuroimage.2004.06.047}{doi:10.1016/j.neuroimage.2004.06.047}
+#'     
 #' @examples
 #' \dontrun{
 #' # note that we override the default NeuronName field
@@ -22,6 +50,14 @@
 #' fileformats(class='neuron', read=TRUE)
 #' }
 read.neuron<-function(f, format=NULL, ...){
+  if(grepl("^http[s]{0,1}://", f)) {
+    url=f
+    # download remote url to local file in tempdir
+    f=file.path(tempdir(), basename(f))
+    on.exit(unlink(f))
+    filecontents=httr::GET(url)
+    writeBin(httr::content(filecontents,type = 'raw'), con = f)
+  } else url=NULL
   #if(!file.exists(f)) stop("Unable to read file: ",f)
   if(is.null(format))
     format=tolower(sub(".*\\.([^.]+$)","\\1",basename(f)))
@@ -32,9 +68,18 @@ read.neuron<-function(f, format=NULL, ...){
     if(length(objname)>1) stop("More than 1 object in file:",f)
     n=get(objname,envir=environment())
   } else {
-    ffs=getformatreader(f)
-    if(is.null(ffs)) stop("Unable to identify file type of:", f)
-    n=match.fun(ffs$read)(f, ...)
+    ffs=getformatreader(f, class = 'neuron')
+    if(is.null(ffs)) {
+      # as a special test, check to see if this looks like an swc file
+      # we don't do this by default since is.swc is a little slow
+      if(is.swc(f)) n=read.neuron.swc(f, ...)
+      else stop("Unable to identify file type of:", f)
+    } else {
+      n=match.fun(ffs$read)(f, ...)
+    }
+  }
+  if(!is.null(url)){
+    attr(n, 'url')=url
   }
   # make sure that neuron actually inherits from neuron
   # we can rely on dotprops/neuronlist objects to have the correct class
@@ -46,26 +91,28 @@ read.neuron<-function(f, format=NULL, ...){
 #' @details This function will cope with the same set of file formats offered by
 #'   \code{read.neuron}.
 #'   
-#'   If the \code{paths} argument specifies a (single) directory then all files
-#'   in that directory will be read unless an optional regex pattern is also
-#'   specified.
+#'   If the \code{paths} argument specifies a (single) directory then all files 
+#'   in that directory will be read unless an optional regex pattern is also 
+#'   specified. Similarly, if \code{paths} specifies a zip archive, all neurons
+#'   within the archive will be loaded.
 #'   
-#'   \code{neuronnames} must specify a unique set of names that will be used as
+#'   \code{neuronnames} must specify a unique set of names that will be used as 
 #'   the names of the neurons in the resultant neuronlist. If \code{neuronnames}
-#'   is a a function then this will be applied to the path to each neuron. The
-#'   default value is the function \code{basename} which results in each neuron
+#'   is a a function then this will be applied to the path to each neuron. The 
+#'   default value is the function \code{basename} which results in each neuron 
 #'   being named for the input file from which it was read.
 #'   
-#'   The optional dataframe (\code{df}) detailing each neuron should have
-#'   \code{rownames} that match the names of each neuron. It would also make
-#'   sense if the same key was present in a column of the data frame. If the
+#'   The optional dataframe (\code{df}) detailing each neuron should have 
+#'   \code{rownames} that match the names of each neuron. It would also make 
+#'   sense if the same key was present in a column of the data frame. If the 
 #'   dataframe contains more rows than neurons, the superfluous rows are dropped
-#'   with a warning. If the dataframe is missing rows for some neurons an error
-#'   is generated. If SortOnUpdate is TRUE then updating an existing neuronlist
-#'   should result in a new neuronlist with ordering identical to reading all
+#'   with a warning. If the dataframe is missing rows for some neurons an error 
+#'   is generated. If SortOnUpdate is TRUE then updating an existing neuronlist 
+#'   should result in a new neuronlist with ordering identical to reading all 
 #'   neurons from scratch.
 #' @param paths Paths to neuron input files \emph{or} a directory containing 
-#'   neurons \emph{or} a \code{\link{neuronlistfh}} object.
+#'   neurons \emph{or} a \code{\link{neuronlistfh}} object, \emph{or} a zip 
+#'   archive containing multiple neurons.
 #' @param pattern If paths is a directory, \link{regex} that file names must 
 #'   match.
 #' @param neuronnames Character vector or function that specifies neuron names. 
@@ -81,10 +128,25 @@ read.neuron<-function(f, format=NULL, ...){
 #' @export
 #' @seealso \code{\link{read.neuron}}
 #' @family neuronlist
+#' @examples
+#' \dontrun{
+#' ## Read C. elegans neurons from OpenWorm github repository
+#' vds=paste0("https://raw.githubusercontent.com/openworm/CElegansNeuroML/",
+#'   "103d500e066125688aa7ac5eac7e9b2bb4490561/CElegans/generatedNeuroML/VD",
+#'   1:13,".morph.xml")
+#' vdnl=read.neurons(vds)
+#' plot3d(vdnl)
+#' }
 read.neurons<-function(paths, pattern=NULL, neuronnames=basename, format=NULL,
                        nl=NULL, df=NULL, OmitFailures=TRUE, SortOnUpdate=FALSE,
                        ...){
-  if(inherits(paths,'neuronlistfh')){
+  if(length(paths) == 1 && grepl("\\.zip$", paths)) {
+    neurons_dir <- file.path(tempfile(pattern = "user_neurons"))
+    on.exit(unlink(neurons_dir, recursive=TRUE))
+    unzip(paths, exdir=neurons_dir)
+    paths=dir(neurons_dir, full.names = TRUE, recursive=TRUE)
+  }
+  else if(inherits(paths,'neuronlistfh')){
     if(!inherits(attr(paths,'db'),'filehashRDS'))
       stop("read.neurons only supports reading neuronlistfh with an RDS format filehash")
     nlfh=paths
@@ -207,7 +269,7 @@ read.neurons<-function(paths, pattern=NULL, neuronnames=basename, format=NULL,
 #'   according to the value of rval.
 #'   
 #'   \item \code{getformatreader} returns a list. The reader can be accessed 
-#'   with \code{$read}
+#'   with \code{$read} and the format can be acessed by \code{$format}.
 #'   
 #'   \item \code{getformatwriter} returns a list. The writer can be accessed 
 #'   with \code{$write}.}
@@ -239,7 +301,7 @@ fileformats<-function(format=NULL,ext=NULL,read=NULL,write=NULL,class=NULL,
     if(!is.null(ext) && !is.na(ext)){
       if(substr(ext,1,1)!=".") ext=paste(".",sep="",ext)
       currentformats<-Filter(function(x) isTRUE(
-        get(x,envir=.fileformats)$ext%in%ext), currentformats)
+        ext%in%get(x,envir=.fileformats)$ext), currentformats)
     }
   }
   rval=match.arg(rval)
@@ -279,7 +341,7 @@ registerformat<-function(format=NULL,ext=format,read=NULL,write=NULL,magic=NULL,
   if(is.null(read) && is.null(write)) 
     stop("Must provide at least one read or write function")
   
-  if(substr(ext,1,1)!=".") ext=paste(".",sep='',ext)
+  ext=sub("^([^.])",".\\1",ext)
   
   assign(format,list(ext=ext,read=read,write=write,magic=magic,magiclen=magiclen,
                      class=class),
@@ -287,7 +349,7 @@ registerformat<-function(format=NULL,ext=format,read=NULL,write=NULL,magic=NULL,
   invisible()
 }
 
-#' @description \code{getformatwriter} gets the function to read a file
+#' @description \code{getformatreader} gets the function to read a file
 #' @rdname fileformats
 #' @param file Path to a file
 #' @details \code{getformatreader} starts by reading a set number of bytes from 
@@ -295,6 +357,11 @@ registerformat<-function(format=NULL,ext=format,read=NULL,write=NULL,magic=NULL,
 #'   magic functions to see if it can identify the file. Presently formats are 
 #'   in a queue in alphabetical order, dispatching on the first match.
 #' @export
+#' @examples
+#' swc=tempfile(fileext = '.swc')
+#' write.neuron(Cell07PNs[[1]], swc)
+#' stopifnot(isTRUE(getformatreader(swc)$format=='swc'))
+#' unlink(swc)
 getformatreader<-function(file, class=NULL){
   formatsforclass<-fileformats(class=class)
   if(!length(formatsforclass)) return(NULL)
@@ -315,6 +382,7 @@ getformatreader<-function(file, class=NULL){
   ext=tolower(sub(".*(\\.[^.]+$)","\\1",basename(file)))
   for(format in formatsforclass){
     ffs=get(format,envir=.fileformats)
+    ffs$format=format
     
     # check that we have a read function for this format
     if (!"read"%in%names(ffs)) next
@@ -331,18 +399,33 @@ getformatreader<-function(file, class=NULL){
 }
 
 #' @description \code{getformatwriter} gets the function to write a file
-#' @details If \code{ext=NA} then extension will not be used to query file 
-#'   formats and it will be overwritten by the default extensions returned by 
-#'   \code{fileformats}. If \code{ext='.someext'} \code{getformatwriter} will 
-#'   use the specified extension to overwrite the value returned by 
-#'   \code{fileformats}. If \code{ext=NULL} and 
-#'   \code{file='somefilename.someext'} then \code{ext} will be set to 
-#'   \code{'someext'} and that will override the value returned by 
-#'   \code{fileformats}. If \code{file='somefile_without_extension'} then the
-#'   suppplied or calculated extension will be appended to \code{file}. See
-#'   \code{\link{write.neuron}} for code to make this discussion more concrete.
+#' @section getformatwriter output file: If \code{getformatwriter} is passed a 
+#'   \code{file} argument, it will be processed based on the registered 
+#'   fileformats information and the \code{ext} argument to give a final output 
+#'   path in the \code{$file} element of the returned \code{list}.
+#'   
+#'   If \code{ext='.someext'} \code{getformatwriter} will use the specified 
+#'   extension to overwrite the default value returned by \code{fileformats}.
+#'   
+#'   If \code{ext=NULL}, the default, and \code{file='somefilename.someext'}
+#'   then \code{file} will be untouched and \code{ext} will be set to
+#'   \code{'someext'} (overriding the value returned by \code{fileformats}).
+#'   
+#'   If \code{file='somefile_without_extension'} then the suppplied or 
+#'   calculated extension will be appended to \code{file}.
+#'   
+#'   If \code{ext=NA} then the input \code{file} name will not be touched (even 
+#'   if it has no extension at all).
+#'   
+#'   Note that if \code{ext=NULL} or \code{ext=NA}, then only the specified 
+#'   format or, failing that, the \code{file} extension will be used to query 
+#'   the fileformats database for a match.
+#'   
+#'   See \code{\link{write.neuron}} for code to make this discussion more 
+#'   concrete.
 #' @rdname fileformats
 #' @export
+#' @seealso \code{\link{write.neuron}}
 getformatwriter<-function(format=NULL, file=NULL, ext=NULL, class=NULL){
   
   if(!is.null(file) && is.null(ext)){
@@ -357,13 +440,18 @@ getformatwriter<-function(format=NULL, file=NULL, ext=NULL, class=NULL){
   r=nfs[[1]]
   
   if(ext_was_set) r$ext=ext
+  # Process file name if one was supplied
   if(!is.null(file)) {
-    r$file=if(nzchar(tools::file_ext(file))){
-      # the file we were given has an extension
-      sub("\\.[^.]+$", r$ext, file)
+    r$file=if(length(ext) && is.na(ext)) {
+      # ext=NA, don't touch file name!
+      file
+    } else if(nzchar(tools::file_ext(file))){
+      # the file we were given has an extension, replace it
+      sub("\\.[^.]+$", r$ext[1], file)
     } else {
-      # the input file did not have an extension so just append
-      paste0(file, r$ext)
+      # the input file did not have an extension
+      # append if ext was anything other than NA
+      paste0(file, r$ext[1])
     }
   }
   r
@@ -371,7 +459,7 @@ getformatwriter<-function(format=NULL, file=NULL, ext=NULL, class=NULL){
 
 #' Read a neuron in swc file format
 #' 
-#' This function should normally only be called from read.neuron and is not
+#' This function should normally only be called from read.neuron and is not 
 #' designed for use by end users.
 #' @section SWC Format: According to 
 #'   \url{http://www.soton.ac.uk/~dales/morpho/morpho_doc} SWC file format has a
@@ -379,6 +467,7 @@ getformatwriter<-function(format=NULL, file=NULL, ext=NULL, class=NULL){
 #' @param f path to file
 #' @param ... Additional arguments passed to \code{as.neuron()} and then on to 
 #'   \code{neuron()}
+#' @seealso \code{\link{is.swc}}
 read.neuron.swc<-function(f, ...){
   ColumnNames<-c("PointNo","Label","X","Y","Z","W","Parent")
   d=read.table(f, header = FALSE, sep = "", quote = "\"'", dec = ".",
@@ -390,20 +479,63 @@ read.neuron.swc<-function(f, ...){
   as.neuron(d, InputFileName=f, ...)
 }
 
+#' Test if a file is an SWC format neuron
+#' 
+#' @details Note that this test is somewhat expensive compared with the other 
+#'   file tests since SWC files do not have a consistent magic value. It
+#'   therefore often has to read and parse the first few lines of the file in 
+#'   order to determine whether they are consistent with the SWC format.
+#' @param f Path to one or more files
+#' @inheritParams is.nrrd
+#' @return logical value
+#' @seealso \code{\link{read.neuron}}
+#' @export
+is.swc<-function(f, TrustSuffix=TRUE) {
+  if(length(f)>1) return(sapply(f, is.swc, TrustSuffix))
+  if(TrustSuffix && tools::file_ext(f)=='.swc') return(TRUE)
+  
+  first_char=readChar(f, nchars = 1, useBytes = TRUE)
+  
+  # must start with a number or a comment character
+  if(!grepl(first_char, "#0123456789", fixed = TRUE, useBytes = T))
+    return(FALSE)
+  
+  # try reading a line, treating any warnings as errors
+  first_line=try(withCallingHandlers(
+    read.table(f, nrows = 1, col.names = 
+                 c("PointNo","Label","X","Y","Z","W","Parent")),
+    warning=function(w) stop(w) ),
+    silent = TRUE)
+  if(inherits(first_line, 'try-error')) return(FALSE)
+  
+  # these columns must be integer
+  if(!all(sapply(first_line[c("PointNo", "Label", "Parent")], is.integer)))
+    return(FALSE)
+  # final check these columns must be numeric (double or integer)
+  all(sapply(first_line[c("X", "Y", "Z", "W")], is.numeric))
+}
+
 #' Write out a neuron in any of the file formats we know about
 #' 
-#' If file is not specified the neuron's InputFileName field will be checked
-#' (for a dotprops object it will be the \code{'file'} attribute). If this is
-#' missing there will be an error. If dir is specified it will be combined with
-#' basename(file). If file is specified but format is not, it will be inferred
+#' If file is not specified the neuron's InputFileName field will be checked 
+#' (for a dotprops object it will be the \code{'file'} attribute). If this is 
+#' missing there will be an error. If dir is specified it will be combined with 
+#' basename(file). If file is specified but format is not, it will be inferred 
 #' from file's extension.
+#' 
+#' @details Note that if \code{file} does not have an extension then the default
+#'   extension for the specified \code{format} will be appended. This behaviour
+#'   can be suppressed by setting \code{ext=NA}.
+#'   
 #' @param n A neuron
 #' @param file Path to output file
 #' @param dir Path to directory (this will replace dirname(file) if specified)
 #' @param format Unique abbreviation of one of the registered file formats for 
 #'   neurons including 'swc', 'hxlineset', 'hxskel'
 #' @param ext Will replace the default extension for the filetype and should 
-#'   include the period eg \code{ext='.amiramesh'} or \code{ext='_reg.swc'}
+#'   include the period eg \code{ext='.amiramesh'} or \code{ext='_reg.swc'}. The
+#'   special value of ext=NA will prevent the extension from being changed or 
+#'   added e.g. if the desired file name does not have an extension.
 #' @param Force Whether to overwrite an existing file
 #' @param MakeDir Whether to create directory implied by \code{file} argument.
 #' @param ... Additional arguments passed to selected writer function
@@ -428,9 +560,10 @@ write.neuron<-function(n, file=NULL, dir=NULL, format=NULL, ext=NULL,
     format='rds'
     if(is.null(file)) {
       file=basename(attr(n,"file"))
-      # don't use the extension of file attribute to override default extension 
-      # returned by query fileformats registry
-      if(is.null(ext)) ext=NA
+      if(is.null(file))
+        stop("No file specified and dotprops obj does not have a file attribute")
+      # trim off the suffix unless we have specified ext=NA 
+      if(!(length(ext) && is.na(ext))) file=tools::file_path_sans_ext(file)
     }
   }
   if(is.null(file)){
@@ -438,9 +571,9 @@ write.neuron<-function(n, file=NULL, dir=NULL, format=NULL, ext=NULL,
     file=basename(n$InputFileName)
     if(is.null(file))
       stop("No file specified and neuron does not have an InputFileName")
-    # don't use the extension of InputFileName to override default extension 
-    # returned by query fileformats registry
-    if(is.null(ext)) ext=NA
+    # trim off the suffix if ext!=NA
+    if(!(length(ext) && is.na(ext)))
+      file=tools::file_path_sans_ext(file)
   }
   fw=getformatwriter(format=format, file=file, ext=ext, class='neuron')
   file=fw$file
@@ -485,23 +618,36 @@ write.neuron.swc<-function(x, file, ...){
   write.table(df, file, col.names=F, row.names=F, append=TRUE, ...)
 }
 
-#' Write neurons from a neuronlist object to individual files
+#' Write neurons from a neuronlist object to individual files, or a zip archive
 #' 
+#' @details See \code{\link{write.neuron}} for details of how to specify the 
+#'   file format/extension/name of the output files and how to establish what 
+#'   output file formats are available. A zip archive of files can be written by
+#'   specifying a value of \code{dir} that ends in \code{.zip}.
 #' @param nl neuronlist object
-#' @param dir directory to write neurons
+#' @param dir directory to write neurons, or path to zip archive (see Details).
+#' @inheritParams write.neuron
 #' @param subdir String naming field in neuron that specifies a subdirectory OR 
 #'   expression to evaluate in the context of neuronlist's df attribute
 #' @param INDICES Character vector of the names of a subset of neurons in 
 #'   neuronlist to write.
-#' @param files Character vector or expression specifying output filenames. See
+#' @param files Character vector or expression specifying output filenames. See 
 #'   examples and \code{\link{write.neuron}} for details.
-#' @param ... Additional arguments passed to write.neuron
+#' @param ... Additional arguments passed to \code{\link{write.neuron}}
+#' @inheritParams write.neuron
+#' @return the path to the output file(s), absolute when this is a zip file.
 #' @author jefferis
 #' @export
 #' @seealso \code{\link{write.neuron}}
 #' @family neuronlist
 #' @examples
 #' \dontrun{
+#' # write some neurons in swc format
+#' write.neurons(Cell07PNs, dir="testwn", format='swc')
+#' # write some neurons in Amira hxlineset format
+#' write.neurons(Cell07PNs, dir="testwn", format='hxlineset')
+#' 
+#' # organise new files in directory hierarchy by glomerulus and Scored.By field
 #' write.neurons(Cell07PNs,dir="testwn",
 #'   subdir=file.path(Glomerulus,Scored.By),format='hxlineset')
 #' # ensure that the neurons are named according to neuronlist names
@@ -518,10 +664,27 @@ write.neuron.swc<-function(x, file, ...){
 #' write.neurons(subset(Cell07PNs, Scored.By="ACH"),dir="testwn4",
 #'   subdir=Glomerulus, files=paste0(ID,'.am'), format='hxlineset')
 #' }
-write.neurons<-function(nl, dir, subdir=NULL, INDICES=names(nl), files=NULL, ...){
+write.neurons<-function(nl, dir, format=NULL, subdir=NULL, INDICES=names(nl), 
+                        files=NULL, Force=FALSE, ...){
+  if(grepl("\\.zip", dir)) {
+    zip_file=dir
+    # check if file exists (and we want to overwrite)
+    if(file.exists(zip_file)){
+      if(!Force)
+        stop("Zip file: ", zip_file, "already exists")
+      unlink(zip_file)
+    }
+    # Get absolute path of parent dir
+    zip_dir=tools::file_path_as_absolute(dirname(zip_file))
+    # ... and use that to construct absolute path to output zip
+    zip_file=file.path(zip_dir, basename(zip_file))
+    dir <- file.path(tempfile("user_neurons"))
+  } else {
+    zip_file=NULL
+  }
   if(!file.exists(dir)) dir.create(dir)
   df=attr(nl,'df')
-  # Construct subdirectory structure based on 
+  # Construct subdirectory structure based on variables in attached data.frame
   ee=substitute(subdir)
   subdirs=NULL
   if(!is.null(ee) && !is.character(ee)){
@@ -548,7 +711,14 @@ write.neurons<-function(nl, dir, subdir=NULL, INDICES=names(nl), files=NULL, ...
       thisdir=subdirs[nn]
     }
     if(!file.exists(thisdir)) dir.create(thisdir, recursive=TRUE)
-    written[nn]=write.neuron(n, dir=thisdir, file = files[nn], ...)
+    written[nn]=write.neuron(n, dir=thisdir, file = files[nn], format=format, Force=Force, ...)
+  }
+  if(!is.null(zip_file)) {
+    owd=setwd(dir)
+    on.exit(setwd(owd))
+    zip(zip_file, files=dir(dir, recursive = TRUE))
+    unlink(dir, recursive=TRUE)
+    written<-zip_file
   }
   invisible(written)
 }
