@@ -61,17 +61,18 @@ as.neuronlist<-function(l, ...) UseMethod("as.neuronlist")
 #'   (see details).
 #' @method as.neuronlist default
 #' @rdname as.neuronlist
-as.neuronlist.default<-function(l, df, AddClassToNeurons=TRUE, ...){
+as.neuronlist.default<-function(l, df=NULL, AddClassToNeurons=TRUE, ...){
   if(is.neuron(l)) {
     n<-l
     l<-list(n)
     names(l)<-n$NeuronName
   }
-  if(!missing(df) &&!is.null(df)) {
+  if(!is.null(df)) {
     if(nrow(df)!=length(l)) 
       stop("data frame must have same number of rows as there are neurons")
     attr(l,"df")=df
-    if(is.null(names(l)))
+    # null or empty string names
+    if(is.null(names(l)) || all(!nzchar(names(l))))
       names(l)=rownames(df)
     else if(any(names(l)!=rownames(df)))
       stop("mismatch between neuronlist names and dataframe rownames")
@@ -246,30 +247,39 @@ nlapply<-function (X, FUN, ..., subset=NULL, OmitFailures=NA){
 }
 
 #' @inheritParams base::mapply
-#' @details Note that for \code{nmapply} the first argument in \dots must be a
-#'   \code{neuronlist}
 #' @rdname nlapply
 #' @seealso \code{\link{mapply}}
 #' @export
-nmapply<-function(FUN, ..., MoreArgs = NULL, SIMPLIFY = FALSE,
-                  USE.NAMES = TRUE, OmitFailures=NA){
-  if(missing(...))
-    stop("First argument in ... must be a neuronlist!")
-  
-  X<-pairlist(...)[[1]]
+nmapply<-function(FUN, X, ..., MoreArgs = NULL, SIMPLIFY = FALSE,
+                  USE.NAMES = TRUE, subset=NULL, OmitFailures=NA){
   if(!is.neuronlist(X))
-    stop("First argument in ... must be a neuronlist!")
+    stop("X must be a neuronlist!")
   cl=if(is.neuronlist(X) && !inherits(X, 'neuronlistfh')) class(X)
   else c("neuronlist",'list')
+  if(!is.null(subset)){
+    if(!is.character(subset)) subset=names(X)[subset]
+    Y=X
+    X=X[subset]
+  }
   
   TFUN = if(is.na(OmitFailures)) FUN else function(...) try(FUN(...), silent=TRUE)
-  rval=structure(mapply(TFUN, ..., MoreArgs = MoreArgs, SIMPLIFY = SIMPLIFY,
+  rval=structure(mapply(TFUN, X, ..., MoreArgs = MoreArgs, SIMPLIFY = SIMPLIFY,
                         USE.NAMES = USE.NAMES), class=cl, df=attr(X, 'df'))
-  if(isTRUE(OmitFailures)){
+  if(isTRUE(OmitFailures))
     failures=sapply(rval, inherits, 'try-error')
-    if(any(failures)) rval=rval[!failures]
+
+  if(is.null(subset)){
+    if(isTRUE(OmitFailures) && any(failures)) rval[!failures]
+    else rval
+  } else {
+    if(isTRUE(OmitFailures) && any(failures)){
+      Y[subset[!failures]]=rval[!failures]
+      Y=Y[setdiff(names(Y),subset[failures])]
+    } else {
+      Y[subset]=rval
+    }
+    Y
   }
-  rval
 }
 
 #' 3D plots of the elements in a neuronlist, optionally using a subset 
@@ -382,15 +392,16 @@ plot3d.neuronlist<-function(x, subset, col=NULL, colpal=rainbow, skipRedraw=200,
 #' @rdname plot3d.neuronlist
 #' @method plot3d character
 #' @export
-#' @param db A neuronlist to use as the source of objects to plot. If missing,
-#'   defaults to neuronlist specified by options('nat.default.neuronlist')
+#' @param db A neuronlist to use as the source of objects to plot. If
+#'   \code{NULL}, the default, will use the neuronlist specified by
+#'   options('nat.default.neuronlist')
 #' @description \code{plot3d.character} is a convenience method intended for 
 #'   exploratory work on the command line.
 #' @details plot3d.character will check if options('nat.default.neuronlist') has
 #'   been set and then use x as an identifier to find a neuron in that 
 #'   neuronlist.
-plot3d.character<-function(x, db, ...) {
-  if(missing(db))
+plot3d.character<-function(x, db=NULL, ...) {
+  if(is.null(db))
     db=get(getOption('nat.default.neuronlist',
                      default=stop('Option "nat.default.neuronlist" is not set.",
                                   " See ?nat for details.')))
@@ -412,7 +423,7 @@ plot3d.character<-function(x, db, ...) {
 #' @param x a neuron list or, for \code{plot3d.character}, a character vector of
 #'   neuron names. The default neuronlist used by plot3d.character can be set by
 #'   using \code{options(nat.default.neuronlist='mylist')}. See 
-#'   ?\code{\link{nat}} for details. \code{\link{nat-package}}.
+#'   ?\code{\link{nat}} for details.
 #' @param col An expression specifying a colour evaluated in the context of the 
 #'   dataframe attached to nl (after any subsetting). See details.
 #' @param add Logical specifying whether to add data to an existing plot or make
@@ -674,9 +685,11 @@ subset.neuronlist<-function(x, subset, filterfun,
 #' @param sel3dfun A \code{\link{select3d}} style function to indicate if points
 #'   are within region
 #' @param indices Names of neurons to search (defaults to all neurons in list)
-#' @param db \code{neuronlist} to search. Can also be a character vector naming
+#' @param db \code{neuronlist} to search. Can also be a character vector naming 
 #'   the neuronlist. Defaults to \code{options('nat.default.neuronlist')}.
 #' @param threshold More than this many points must be present in region
+#' @param invert Whether to return neurons outside the selection box (default
+#'   \code{FALSE})
 #' @return Character vector of names of selected neurons
 #' @export
 #' @seealso \code{\link{select3d}, \link{subset.neuronlist}}
@@ -694,7 +707,8 @@ subset.neuronlist<-function(x, subset, filterfun,
 #' find.neuron(Negate(vertical_lobe), db=kcs20)
 #' }
 find.neuron<-function(sel3dfun=select3d(), indices=names(db), 
-                      db=getOption("nat.default.neuronlist"), threshold=0){
+                      db=getOption("nat.default.neuronlist"), threshold=0,
+                      invert=FALSE){
   
   if(is.null(db))
     stop("Please pass a neuronlist in argument db or set options",
@@ -704,5 +718,168 @@ find.neuron<-function(sel3dfun=select3d(), indices=names(db),
     pointsinside=sel3dfun(xyzmatrix(x))
     sum(pointsinside, na.rm=T)>threshold
   }
-  subset(db, subset=indices, filterfun=selfun, rval='names')
+  sel_neurons=subset(db, subset=indices, filterfun=selfun, rval='names')
+  if(invert) setdiff(indices, sel_neurons) else sel_neurons
+}
+
+#' Find neurons with soma inside 3d selection box (usually drawn in rgl window)
+#' 
+#' @details Can work on \code{neuronlist}s containing \code{neuron} objects 
+#'   \emph{or} \code{neuronlist}s whose attached data.frame contains soma 
+#'   positions specified in columns called X,Y,Z  .
+#' @inheritParams find.neuron
+#' @return Character vector of names of selected neurons
+#' @export
+#' @seealso \code{\link{select3d}, \link{subset.neuronlist}, \link{find.neuron}}
+find.soma <- function (sel3dfun = select3d(), indices = names(db), 
+                       db = getOption("nat.default.neuronlist"),
+                       invert=FALSE) 
+{
+  if (is.null(db)) 
+    stop("Please pass a neuronlist in argument db or set options", 
+         "(nat.default.neuronlist='myfavneuronlist'). See ?nat for details.")
+  if (is.character(db)) 
+    db = get(db)
+  df=attr(db, 'df')
+  sel_neurons=if(all(c("X","Y","Z") %in% colnames(df))){
+    # assume these represent soma position
+    somapos=df[indices,c("X","Y","Z")]
+    indices[sel3dfun(somapos)]
+  } else {
+    selfun = function(x) {
+      somapos=x$d[x$StartPoint, c("X","Y","Z")]
+      isTRUE(sel3dfun(somapos))
+    }
+    subset(db, subset = indices, filterfun = selfun, rval = "names")
+  }
+  if(invert) setdiff(indices, sel_neurons) else sel_neurons
+}
+
+
+#' Scan through a set of neurons, individually plotting each one in 3D
+#' 
+#' Can also choose to select specific neurons along the way and navigate 
+#' forwards and backwards.
+#' 
+#' @param neurons a \code{neuronlist} object or a character vector of names of 
+#'   neurons to plot from the neuronlist specified by \code{db}.
+#' @inheritParams plot3d.character
+#' @param col the color with which to plot the neurons (default \code{'red'}).
+#' @param Verbose logical indicating that info about each selected neuron should
+#'   be printed (default \code{TRUE}).
+#' @param Wait logical indicating that there should be a pause between each 
+#'   displayed neuron.
+#' @param sleep time to pause between each displayed neuron when 
+#'   \code{Wait=TRUE}.
+#' @param extrafun an optional function called when each neuron is plotted, with
+#'   two arguments: the current neuron name and the current \code{selected} 
+#'   neurons.
+#' @param selected_file an optional path to a \code{yaml} file that already 
+#'   contains a selection.
+#' @param selected_col the color in which selected neurons (such as those 
+#'   specified in \code{selected_file}) should be plotted.
+#' @param yaml a logical indicating that selections should be saved to disk in 
+#'   (human-readable) \code{yaml} rather than (machine-readable) \code{rda} 
+#'   format.
+#' @param ... extra arguments to pass to \code{\link{plot3d}}.
+#'   
+#' @return A character vector of names of any selected neurons, of length 0 if 
+#'   none selected.
+#' @importFrom yaml yaml.load_file as.yaml
+#' @seealso \code{\link{plot3d.character}}, \code{\link{plot3d.neuronlist}}
+#' @export
+#' @examples
+#' \dontrun{
+#' # scan a neuronlist
+#' nlscan(kcs20)
+#' 
+#' # using neuron names
+#' nlscan(names(kcs20), db=kcs20)
+#' # equivalently using a default neuron list
+#' options(nat.default.neuronlist='kcs20')
+#' nlscan(names(kcs20))
+#' }
+#' # scan without waiting
+#' nlscan(kcs20[1:4], Wait=FALSE, sleep=0)
+#' \dontrun{
+#' # could select e.g. the gamma neurons with unbranched axons
+#' gammas=nlscan(kcs20)
+#' clear3d()
+#' plot3d(kcs20[gammas])
+#' 
+#' # plot surface model of brain first
+#' # nb depends on package only available on github
+#' devtools::install_github(username = "jefferislab/nat.flybrains")
+#' library(nat.flybrains)
+#' plot3d(FCWB)
+#' # could select e.g. the gamma neurons with unbranched axons
+#' gammas=nlscan(kcs20)
+#' clear3d()
+#' plot3d(kcs20[gammas])
+#' }
+nlscan <- function(neurons, db=NULL, col='red', Verbose=T, Wait=T, sleep=0.1,
+                   extrafun=NULL, selected_file=NULL, selected_col='green',
+                   yaml=TRUE, ...) {
+  if(is.neuronlist(neurons)) {
+    db=neurons
+    neurons=names(db)
+  }
+  frames <- length(neurons)
+  if(length(col)==1) col <- rep(col,frames)
+  selected <- character()
+  i <- 1
+  if(!is.null(selected_file) && file.exists(selected_file)) {
+    selected <- yaml.load_file(selected_file)
+    if(!all(names(selected) %in% neurons)) stop("Mismatch between selection file and neurons.")
+  }
+  
+  savetodisk <- function(selected, selected_file) {
+    if(is.null(selected_file)) selected_file <- file.choose(new=TRUE)
+    if(yaml){
+      if(!grepl("\\.yaml$",selected_file)) selected_file <- paste(selected_file,sep="",".yaml")
+      message("Saving selection to disk as ", selected_file, ".")
+      writeLines(as.yaml(selected), con=selected_file)
+    } else {
+      if(!grepl("\\.rda$", selected_file)) selected_file <- paste(selected_file, sep="", ".rda")
+      save(selected, file=selected_file)
+      message("Saving selection to disk as ", selected_file)
+    }
+    selected_file
+  }
+  chc <- NULL
+  while(TRUE){
+    if(i > length(neurons) || i < 1) break
+    n <- neurons[i]
+    cat("Current neuron:", n, "(", i, "/", length(neurons), ")\n")
+    pl <- plot3d(n, db=db, col=substitute(ifelse(n %in% selected, selected_col, col[i])), ..., SUBSTITUTE=FALSE)
+    # call user supplied function
+    more_rgl_ids <- list()
+    if(!is.null(extrafun))
+      more_rgl_ids <- extrafun(n, selected=selected)
+    if(Wait){
+      chc <- readline("Return to continue, b to go back, s to select, d [save to disk], t to stop, c to cancel (without returning a selection): ")
+      if(chc=="c" || chc=='t'){
+        sapply(pl, rgl.pop, type='shape')
+        sapply(more_rgl_ids, rgl.pop, type='shape')
+        break
+      }
+      if(chc=="s") {
+        if(n %in% selected) {
+          message("Deselected: ", n)
+          selected <- setdiff(selected, n)
+        } else selected <- union(selected, n)
+      }
+      if(chc=="b") i <- i-1
+      else if (chc=='d') savetodisk(selected, selected_file)
+      else i <- i+1
+    } else {
+      Sys.sleep(sleep)
+      i <- i+1
+    }
+    sapply(pl, rgl.pop, type='shape')
+    sapply(more_rgl_ids, rgl.pop, type='shape')
+  }
+  if(is.null(chc) || chc=='c') return(NULL)
+  if(!is.null(selected_file)) savetodisk(selected, selected_file)
+  selected
 }
